/*********************************************************************/
/* file: action.c - funtions related to the action command           */
/*                             HUT TINTIN                            */
/*                coded by Ossi Tulonen from TinTin III              */
/*                TinTin III coded by peter unold 1992               */
/*********************************************************************/
/*
 * TinTin III code Copyright (c) 1992 Peter Unold
 * All changed code from TinTin III Copyright (c) 1992 1993 1994 Ossi tulonen
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "structs.h"
#include "action.h"
#include "llist.h"
#include "parse.h"
#include "screen.h"
#include "utility.h"
#include "utils.h"

extern struct master_table_data master_table;
extern unsigned char vars[10][BUFFER_SIZE]; /* the $0, $1, $2,....$9 variables */
extern unsigned char text_start, text_end, tintin_retchar;

int doing_var_number = -1;

/***********************/
/* the #action command */
/***********************/

session *action_command(const unsigned char *arg, session *ses)
{
  unsigned char left[BUFFER_SIZE], right[BUFFER_SIZE], result[BUFFER_SIZE/2];
  unsigned char pritext[BUFFER_SIZE], *tmp;
  int found, amount, pos, ret_value, pri = 5;
  struct listnode **myactions, *ln;
  
  myactions = &ses->actions;
  
  arg = get_arg_stop_spaces(arg, left);
  arg = get_arg_stop_spaces(arg, right);
  arg = get_arg_with_spaces(arg, pritext);
  if(*pritext) {
    pri = string_to_number(pritext, ses);
    if(pri < 0 || pri > 20) {
      ses->print((const uchar *)"#Priority must be 0-20!! (using default value)\n\r");
      pri = 5;
    }
  }
  
  if(!*left) {
    ses->print((const uchar *)"#THESE ACTIONS (and substitutes) HAS BEEN DEFINED:\n\r");
    show_list(*myactions, ses);
  }
  
  else if(*left && !*right) {
    for(found = FALSE,ln = searchnode_list2(*myactions, left); ln;
	ln = searchnode_list2(ln->next, left)) {
      if(ln->type == TYPE_ACTION) {
	found = TRUE;
	ln->show(ses);
      }
    }
    if(!found)
      ses->print((const uchar *)"#THAT ACTION IS NOT DEFINED.\n\r");
  }
  
  else {
    if((ln = searchnode_list(*myactions, left)) != NULL) {
      if(ln->type == TYPE_ACTION)
	delete ln;
      else if((ln = searchnode_list(ln->next, left)) != NULL)
	delete ln;
    }

    ln = new listnode(myactions, left, right, TYPE_ACTION, pri);
    if(IS_SET(ses->message_flags, MESSAGE_ACTION))
      ses->print((const uchar *)"#OK. ACTION DEFINED.\n\r");
    
    amount = search_vars(right, result);
    found = FALSE;
    for(pos = 0; pos < amount && !found; pos++) {
      for(tmp = left; *tmp; tmp++)
	if((ret_value = is_variable(tmp)) != 0 &&
	   ((ret_value < 0 && *(tmp + 1) == result[pos]) ||
	    (ret_value > 0 && *(tmp + 3) == result[pos])))
	  break;
      
      if(!*tmp)
	found = TRUE;
    }
    if(found) {
      ses->print((const uchar *)"#Atleast one variable will be replaces with random text when trickered.\n\r");
    }
  }
  return(ses);
}

/*************************/
/* the #unaction command */
/*************************/

session *unaction_command(const unsigned char *arg, session *ses)
{
  unsigned char left[BUFFER_SIZE];
  int found;
  struct listnode **myactions, *ln, *next;

  myactions = &ses->actions;

  arg = get_arg_with_spaces(arg, left);

  for(found = FALSE,ln = searchnode_list2(*myactions, left); ln;
      ln = searchnode_list2(next, left)) {
    next = ln->next;
    if(ln->type == TYPE_ACTION) {
      found = TRUE;
      delete ln;
      if(IS_SET(ses->message_flags, MESSAGE_ACTION))
	ses->print((const uchar *)"#OK. ACTION DELETED\n\r");
    }
  }
  if(!found)
    ses->print((const uchar *)"#THAT ACTION IS NOT DEFINED.\n\r");
  
  return(ses);
}

/*************************************************************************/
/* copy the arg text into the result-space, but substitute the variables */
/* $<name> and if allflag set $0..$9 with the real variables value       */
/* this function return length of result string                          */
/*************************************************************************/
int substitute_vars(const unsigned char *arg, unsigned char *result,
		    int maxlen, int allflag, session *ses, int rem)
{
  unsigned char *org, buf[BUFFER_SIZE], *tmp;
  int n, inside = 0, number, leftspace = maxlen, ignore_flags, wnum;
  struct listnode **myvars, *ln, **mycolors;

  const uchar *str;

  myvars = &ses->variables;
  mycolors = &ses->colors;
  ignore_flags = ses->ignore_flags;

  org = result;
  while(*arg && (leftspace = maxlen - (result - org)) > 0) {
    if(*arg == '$') {
      number = 1;
      while(*(arg + number) == '$')
        number++;

      if(number == inside + 1) {
	if((wnum = is_variable(arg + number - 1))) {
	  
          if(allflag) {
	    if(wnum > 0) {
	      n = *(arg + number + 2) - '0';
	      
	      str = vars[n];
	      while(wnum > 0 && *str) {
		while(*str && isspace(*str))
		  *result++ = *str++;
		
		while(*str && !isspace(*str))
		  *result++ = *str++;
		
		wnum--;
	      }
	      arg += number + 3;
	    } else {
	      n = *(arg + number) - '0';
	      result += str_cat(result, 0, leftspace, vars[n]);
	      arg += number + 1;
	    }
          } else {
            while(*arg == '$')
              *result++ = *arg++;
	    
	    if(wnum > 0) {
	      *result++ = *arg++;          // '-'
	      *result++ = *arg++;          // word amount
	      *result++ = *arg++;          // variable number
	    } else
	      *result++ = *arg++;
          }
        } else if(!IS_SET(ignore_flags, IGNORE_VAR)) {
          tmp = buf;
          arg += number;
          while(*arg && *arg != '$')
            *tmp++ = *arg++;
          *tmp = '\0';
          if(*arg)
            arg++;

	  if((ln = searchnode_list(*myvars, buf)) != NULL) {
	    result += str_cat(result, 0, leftspace, ln->right);
          } else if((ln = searchnode_list(*mycolors, buf)) != NULL) {
	    result += str_cat(result, 0, leftspace, ln->right);
	  } else {
	    ln = new listnode(myvars, buf, (unsigned char *)"", TYPE_VAR, 5);
	    if(IS_SET(ses->message_flags, MESSAGE_VAR))
	      ses->print((const uchar *)"#NEW VARIABLE DEFINED.\n\r");
	  }
        } else {
          while(*arg == '$')
            *result++ = *arg++;

          while(*arg && *arg != '$')
            *result++ = *arg++;

          if(*arg)
            *result++ = *arg++;
        }
      } else {
        while(*arg == '$')
          *result++ = *arg++;

	if((wnum = is_variable(arg - 1))) {
	  if(wnum > 0) {
	    *result++ = *arg++;          // '-'
	    *result++ = *arg++;          // word amount
	    *result++ = *arg++;          // variable number
	  } else
	    *result++ = *arg++;
        } else {
          while(*arg != '$' && *arg)
            *result++ = *arg++;
	  
          if(*arg)
            *result++ = *arg++;
        }
      }
    } else if(*arg == '@') {
      number = 1;
      while(*(arg + number) == '@')
        number++;

      if(number == inside + 1) {
        if(!IS_SET(ignore_flags, IGNORE_CALC)) {
          tmp = buf;
          arg += number;
          while(*arg && *arg != '@')
            *tmp++ = *arg++;
          *tmp = '\0';
          if(*arg)
            arg++;

          sprintf((char *)result, "%d", string_to_number(buf, ses));
          while(*result)
            result++;
        } else {
          while(*arg == '@')
            *result++ = *arg++;

          while(*arg && *arg != '@')
            *result++ = *arg++;

          if(*arg)
            *result++ = *arg++;
        }
      } else {
        while(*arg == '@')
          *result++ = *arg++;
	
	while(*arg != '@' && *arg)
	  *result++ = *arg++;
	
	if(*arg)
	  *result++ = *arg++;
      }
    } else if(*arg == text_start) {
      inside++;
      *result++ = *arg++;
    } else if(*arg == text_end) {
      if(inside)
        inside--;
      *result++ = *arg++;
    } else if(*arg == '\\') {
      if(rem)
	arg++;
      else
	*result++ = *arg++;
      if(*arg)
	*result++ = *arg++;
    } else
      *result++ = *arg++;
  }
  *result = '\0';
  
  return(result - org);
}

// *****************************************
// function to searth all replaced var names
// *****************************************

int search_vars(const unsigned char *arg, unsigned char *result)
{
  int inside = 0, number, pos = 0, wnum;
  
  while(*arg) {
    if(*arg == '$') {
      number = 1;
      while(*(arg + number) == '$')
        number++;

      if(number == inside + 1) {
	if((wnum = is_variable(arg + number - 1))) {
	  if(wnum > 0) {
	    result[pos++] = *(arg + number + 2);
	    arg += number + 3;
	  } else {
	    result[pos++] = *(arg + number);
	    arg += number + 1;
	  }
        } else {
          while(*arg == '$')
            arg++;
	  
          while(*arg && *arg != '$')
            arg++;
	  
          if(*arg)
            arg++;
        }
      } else {
        while(*arg == '$')
          arg++;
	
	if((wnum = is_variable(arg - 1))) {
	  if(wnum > 0)
	    arg += 3;
	  else
	    arg++;
        } else {
          while(*arg != '$' && *arg)
            arg++;
	  
          if(*arg)
            arg++;
        }
      }
    } else if(*arg == '@') {
      number = 1;
      while(*(arg + number) == '@')
        number++;
      
      if(number == inside + 1) {
	arg += number;
	
	while(*arg != '@' && *arg) {
	  if(*arg == '$') {
	    while(*arg && *arg == '$')
	      arg++;
	    
	    if(*arg) {
	      if(isdigit(*arg)) {
		result[pos++] = *arg;
		arg++;
	      } else {
		while(*arg != '$' && *arg != '@' && *arg)
		  arg++;
		
		if(*arg == '$')
		  arg++;
	      }
	    }
	  } else
	    arg++;
	}
	if(*arg)
	  arg++;
      } else {
	arg += number;
	
	while(*arg != '@' && *arg)
	  arg++;
	
	if(*arg)
	  arg++;
      }
    } else if(*arg == text_start) {
      inside++;
      arg++;
    } else if(*arg == text_end) {
      if(inside)
        inside--;
      arg++;
    } else if(*arg == '\\') {
      arg++;
      if(*arg)
	arg++;
    } else
      arg++;
  }
  result[pos] = '\0';
  
  return pos;
}

/******************************************************************/
/* check if a text triggers an action and fill into the variables */
/* return TRUE if triggered                                       */
/******************************************************************/

inline int expanded_char(const unsigned char letter)
{
  if(letter == '\\' ||
     letter == '$' ||
     letter == '@' ||
     letter == text_start ||
     letter == text_end ||
     letter == tintin_retchar)
    return(TRUE);
  return(FALSE);
}

const unsigned char *is_text(const unsigned char *text,
			     const unsigned char *str,
			     const int expand)
{
  unsigned char *var, buf[BUFFER_SIZE], *tmp;
  const unsigned char *tmp2;
  int words, var_num, len;
  
  while(*text && *str) {
    if((words = is_variable(str))) {
      str += 3;
      var_num = (int)((int)*str - (int)'0');
      str++;
      
      if(var_num == doing_var_number)
	doing_var_number = -1;
      var = vars[var_num];
      
      while(isspace(*text))
	text++;
      
      while(words > 0 && *text) {
	if(words == 1) {
	  tmp = buf;
	  len = 0;
	  while(*str && !isspace(*str) && !is_variable(str)) {
	    if(*str == '\\')
	      str++;
	    if(*str) {
	      *tmp++ = *str++;
	      len++;
	    }
	  }
	  *tmp = '\0';
	  
	  tmp2 = text;
	  while(*tmp2 && !isspace(*tmp2))
	    tmp2++;
	  
	  if(tmp2 - text < len) {
	    *var = '\0';
	    return(FALSE);
	  } else {
	    while(text < tmp2 - len) {
	      if(expand && expanded_char(*text))
		*var++ = '\\';
	      *var++ = *text++;
	    }
	    *var = '\0';

	    tmp = buf;
	    while(text < tmp2) {
	      if(*text++ != *tmp++)
		return(FALSE);
	    }
	    if(len == 0) {
	      while(isspace(*str))
		str++;
	      while(isspace(*text))
		text++;
	    }
	  }
	} else {
	  while(*text && !isspace(*text)) {
	    if(expand && expanded_char(*text))
	      *var++ = '\\';
	    *var++ = *text++;
	  }
	  while(isspace(*text))
	    *var++ = *text++;
	}
	words--;
      }
      *var = '\0';

      if(words > 0)
	return(FALSE);
    } else if(*str == '\\') {
      str++;
      if(*str && *text++ != *str++)
	return(FALSE);
    } else {
      if(*text++ != *str++)
	return(FALSE);
    }
  }
  if(*text || (!*text && !*str))
    return(text);
  
  return(FALSE);
}

int check_one_action(const unsigned char *text, const unsigned char *action,
		     const int expand)
{
  unsigned char *str, buf[BUFFER_SIZE], *var, varbuf[BUFFER_SIZE];
  const unsigned char *ret_value;
  int flag, endflag, i;
  
  for(i = 0; i < 10; i++)   // making sure every variable has null string
    *vars[i] = '\0';        // if not found in action text
  
  if(*action == '^') {
    flag = TRUE;
    action++;
  } else
    flag = FALSE;
  
  endflag = FALSE;
  str = buf;
  while(*action && !(*action == '$' && isdigit(*(action + 1)))) {
    if(*action == '^' && !*(action + 1)) {
      endflag = TRUE;
      action++;
    } else
      *str++ = *action++;
  }
  *str = '\0';
  
  if(flag && *buf) {
    if((ret_value = is_text(text, buf, expand))) {
      text = ret_value;
      if(endflag) {
	if(*text)
	  return(FALSE);
	else
	  return(TRUE);
      } else {
	if(!*action)
	  return(TRUE);
      }
    } else
      return(FALSE);
  } else {
    if(endflag) {
      while(*text && (!(ret_value = is_text(text, buf, expand)) || *ret_value))
	text++;
      
      if(!*text)
	return(FALSE);
      else
	return(TRUE);
    } else {
      ret_value = FALSE;
      while(*text && !(ret_value = is_text(text, buf, expand)))
	text++;
      
      if(!ret_value)
	return(FALSE);
      else {
	text = ret_value;
	
	if(!*action)
	  return(TRUE);
      }
    }
  }
  
  while(*text) {
    action++;
    doing_var_number = (int)((int)*action - (int)'0');
    action++;
    
    var = varbuf;
    
    if(!*action) {
      var = vars[doing_var_number];
      if(expand) {
	while(*text) {
	  if(expanded_char(*text))
	    *var++ = '\\';
	  *var++ = *text++;
	}
	*var = '\0';
      } else {
	strcpy((char *)var, (const char *)text);
      }
      return(TRUE);
    }
    str = buf;
    while(*action && (*action != '$' || !isdigit(*(action + 1)))) {
      if(*action == '^' && !*(action + 1)) {
	endflag = TRUE;
	action++;
      } else
	*str++ = *action++;
    }
    *str = '\0';
    
    if(endflag) {
      if(expand) {
	while(*text && (!(ret_value = is_text(text, buf, expand)) ||
			*ret_value)) {
	  if(expanded_char(*text))
	    *var++ = '\\';
	  *var++ = *text++;
	}
      } else {
	while(*text && (!(ret_value = is_text(text, buf, expand)) ||
			*ret_value))
	  *var++ = *text++;
      }
      *var = '\0';
      if(doing_var_number >= 0)
	str_cat(vars[doing_var_number], 0, BUFFER_SIZE, varbuf);
      
      if(!*text)
	return(FALSE);
      else {
	return(TRUE);
      }
    } else {
      ret_value = FALSE;
      if(expand) {
	while(*text && !(ret_value = is_text(text, buf, expand))) {
	  if(expanded_char(*text))
	    *var++ = '\\';
	  *var++ = *text++;
	}
      } else {
	while(*text && !(ret_value = is_text(text, buf, expand)))
	  *var++ = *text++;
      }
      *var = '\0';
      
      if(doing_var_number >= 0)
	str_cat(vars[doing_var_number], 0, BUFFER_SIZE, varbuf);
      
      if(!ret_value) {
	return(FALSE);
      } else {
	text = ret_value;
	if(!*action)
	  return(TRUE);
      }
    }
  }
  
  action++;
  var = vars[*action - '0'];
  action++;
  if(*action) {
    *var = '\0';
    return(TRUE);
  }
  return (FALSE);
}
