#include <sys/types.h>

#include "telnet.h"

typedef unsigned char uchar;

/***********************************************/
/* Some default values you might wanna change: */
/***********************************************/

#define DEFAULT_TINTIN_CHAR '#'       /* tintin char                     */
#define DEFAULT_TEX_START_CHAR '['    /* start text field                */
#define DEFAULT_TEX_END_CHAR ']'      /* end text field                  */
#define DEFAULT_SPEEDWALK TRUE        /* speedwalk                       */
#define DEFAULT_ECHO 0                /* echo                            */
#define DEFAULT_MESS MESSAGE_ACTION | MESSAGE_ALIAS | MESSAGE_VAR | \
                     MESSAGE_WAIT | MESSAGE_SUBS
#define DEFAULT_IGNORE FALSE          /* ignore                          */
#define DEFAULT_CHARS_PER_LINE 80     /* chars per line if WinSize fail  */
#define DEFAULT_LINES_IN_SCR   24     /* lines in screen if WinSize fail */
#define DEFAULT_RETURN_CHAR ';'       /* return char in command lines    */
#define DEFAULT_VERBATIM_CHAR '\\'    /* verbatim char                   */

/**************************************************************************/
/* The stuff below here shouldn't be modified unless you know what you're */
/* doing........                                                          */
/**************************************************************************/

#ifndef TRUE
  #define TRUE 1
#endif

#ifndef FALSE
  #define FALSE 0
#endif

#define DEF_SHELL_ENV "SHELL"
#define DEF_TERM_TYPE (const uchar *)"vt100"

#define BUFFER_SIZE 4096

#define HISTORY_SIZE 100              /* history size            */
#define MAX_PATH_LENGTH 50            /* max path length         */
#define MAX_PROMPT_SIZE 512           /* input prompt max length */
#define MAX_ALIAS_CALLS 10            /* max allowed alias calls */
                                      /* just to check forever loops */
#define MAX_REPEATIF_CALLS 10000      // max allowed repeatif commands
				      // on ONE line.
				      // just to check forever loops
#define WORD_LIST_SIZE  30            /* max words in list       */
#define MAX_SES_HIST    500           /* lines number in memory  */
#define MAX_LINE_LENGTH 1024

#define ESC 27

/* TIMELINEON  should be ansi codes for what kind text is 
   wanted time line to be ...                                      */
#define TIMELINEON  (const uchar *)"\033[7m"    /* reverse video             */

#define ECHO_ACTION 0x1
#define ECHO_IF     0x2
#define ECHO_CMP    0x4
#define ECHO_WAIT   0x8
#define ECHO_ALIAS  0x10

#define IGNORE_ACTION 0x1
#define IGNORE_SUBS   0x2
#define IGNORE_ALIAS  0x4
#define IGNORE_VAR    0x8
#define IGNORE_CALC   0x10

#define MESSAGE_ACTION 0x1
#define MESSAGE_ALIAS  0x2
#define MESSAGE_SUBS   0x4
#define MESSAGE_WAIT   0x8
#define MESSAGE_VAR    0x10

#define NULL_WIN (window *)-1

/************************ structures *********************/

class window;

struct scrollsize {
  int start, end;
};

class hutchar {
public:
  hutchar();
  ~hutchar();
  
  unsigned char *text;
  int len;
};

struct mathnode {
  int next;
  int prev;
  int number;
  struct listnode *var;
  signed char pri;
  signed char opp;
};

class listnode;

struct waitcommdata {
  unsigned char *comm;
  int time;
  struct waitcommdata *next;
};

class session {
public:
  session();
  session(const unsigned char *nam, const unsigned char *add, const int sock);
  ~session();
  
  void print(const unsigned char *text);
  void print_prompt(const int all);
  
  const unsigned char *search_word(const unsigned char *word) const;
  
  void set_autolog(const unsigned char *name);
  const unsigned char *get_autolog() { return autolog; }
  
  // from net.cc file
  void read_net();
  session *write_net(const unsigned char *line);
  session *send_naws();
  session *do_telnet_protecol(int telcom, int telopt);

  
  // from history.cc file
  void session::insert_ses_scrhist(const unsigned char *text, const int flu);
  const unsigned char *search_hist_word(const unsigned char *word, int &amount) const;
  

  const int get_hist_num() const { return session_hist_num; }
  const int get_hist_end() const { return session_hist_end; }
  const unsigned char *get_scrhistory(const int pos) const {
    return history[pos];
  }
  void set_hist_mark(const int pos, const int col) {
    session_hist_mark_pos = pos;
    session_hist_mark_col = col;
  }
  const int get_hist_markp() const { return session_hist_mark_pos; }
  const int get_hist_markc() const { return session_hist_mark_col; }

  const unsigned char *get_name() const { return name; }
  const unsigned char *get_addr() const { return address; }
  window *get_input() const             { return input; }
  void set_input(window *win)           { input = win; }
  window *get_output() const            { return output; }
  void set_output(window *win)          { output = win; }
  const time_t get_time() const         { return start_time; }

  const uchar *get_prompt() const;
  const int get_prompt_len() const;
  void set_prompt(const unsigned char *text);

  const int get_socket() const          { return net; }
  session *get_next() const             { return next; }
  static session *get_sessions()        { return sessions; }

  int echo_flags, ignore_flags, message_flags, term_echoing, line_mode;
  listnode *aliases, *actions, *colors, *variables;
  waitcommdata *waitcomms;
  int snoopstatus;
  int tick_start_time, tickstatus, tick_size, sec_to_tick;
  unsigned char *word_list[WORD_LIST_SIZE];
  unsigned char path[MAX_PATH_LENGTH];
  int path_mark, path_now, path_length;
  FILE *logfile;

private:
  unsigned char *name;
  unsigned char *address, *autolog;
  unsigned char current_line[MAX_LINE_LENGTH], *history[MAX_SES_HIST];
  int session_hist_end, session_hist_num;
  int session_hist_mark_pos, session_hist_mark_col;
  int current_pos;

  hutchar last_text;
  
  unsigned char will_status[NTELOPTS];
  unsigned char do_status[NTELOPTS];
  unsigned char promptbuf[MAX_PROMPT_SIZE];
  int prompt_pos;
  int net;
  time_t start_time;
  window *input, *output;
  
  static session *sessions;
  session *next;
};

struct master_table_data {
  unsigned char *history[HISTORY_SIZE], *clip;
  int history_pos;
  unsigned char inputbuf[BUFFER_SIZE];
  int inputbuf_pos, inputbuf2_pos, cursor_pos, utexrem, ucommand;
  int aliascalls, verbatimmode, no_neg_off;
  window *main_window, *output_window;
  session *main_session;
};
