/*********************************************************************/
/* file: variable.c - funtions related the the variable command      */
/*                            HUT TINTIN                             */
/*                     coded by Ossi Tulonen                         */
/*********************************************************************/
/*
 * Copyright (c) 1992 1993 1994 Ossi tulonen
 * All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */
 
#include <stdio.h>
#include <ctype.h>
#include <string.h>

#include "structs.h"
#include "llist.h"
#include "parse.h"
#include "screen.h"
#include "variable.h"
#include "utility.h"
#include "utils.h"

extern struct master_table_data master_table;
extern unsigned char text_start, text_end;

/*************************/
/* the #variable command */
/*************************/
session *comm_variable(const unsigned char *arg, session *ses)
{
  unsigned char left[BUFFER_SIZE], right[BUFFER_SIZE], buf[BUFFER_SIZE];
  unsigned char word[BUFFER_SIZE], name[BUFFER_SIZE];
  const unsigned char *tmp;
  int removeword = FALSE, verbose = FALSE, first = FALSE, pos, found;
  int second_var = FALSE;
  struct listnode **myvars, **mycolors, *ln;

  myvars = &ses->variables;
  mycolors = &ses->colors;

  arg = get_arg_stop_spaces(arg, left);
  if(*left == '-') {
    tmp = left + 1;
    while(isspace(*tmp))
      tmp++;
    
    while(*tmp) {
      switch(*tmp) {
      case '2':
	second_var = TRUE;
	break;
      case 'f':
      case 'F':
	first = TRUE;
	break;
      case 'r':
      case 'R':
	removeword = TRUE;
	break;
      case 'v':
      case 'V':
	verbose = TRUE;
	break;
      default:
	ses->print((const uchar *)"#Unknown flag in #variable.\n\r");
	break;
      }
      tmp++;
      while(isspace(*tmp))
	tmp++;
    }
    arg = get_arg_stop_spaces(arg, left);
  }
  
  arg = get_arg_with_spaces(arg, word);
  if(second_var) {
    if(!*word) {
      ses->print((const uchar *)"#variable command with -2 flag and no second argument.\n\r");
      return(ses);
    }
    if((ln = searchnode_list(*myvars, word)) != NULL) {
      strcpy((char *)name, (const char *)word);
      strcpy((char *)word, (const char *)ln->right);
      
      if(first) {
	delete ln;
	
	tmp = get_arg_stop_spaces(word, buf); 
	while(*tmp == ' ')
	  tmp++;

	ln = new listnode(myvars, name, tmp, TYPE_VAR, 5);
      }
    } else {
      ses->print((const uchar *)"#No such variable.\n\r");
      return(ses);
    }
  }
  
  if(first) {
    get_arg_stop_spaces(word, right);
  } else {
    strcpy((char *)right, (const char *)word);
  }
  
  if(!*left) {
    ses->print((const uchar *)"#THESE VARIABLES HAS BEEN DEFINED:\n\r");
    show_list(*myvars, ses);
    ses->print((const uchar *)"#THESE COLOR VARIABLES HAS BEEN DEFINED:\n\r");
    show_list(*mycolors, ses);
  } else {
    if((ln = searchnode_list(*mycolors, left)) != NULL)
      delete ln;
    
    if((ln = searchnode_list(*myvars, left)) != NULL) {
      if(removeword) {
	strcpy((char *)word, (const char *)right);
	tmp = ln->right;
	pos = 0;
	found = FALSE;
	do {
	  tmp = get_arg_stop_spaces(tmp, buf);
	  if(strcmp((const char *)word, (const char *)buf)) {
	    if(pos > 0)
	      pos = str_cat(right, pos, sizeof(right), (const uchar *)" ");
	    right[pos++] = text_start;
	    pos = str_cat(right, pos, sizeof(right) , buf);
	    right[pos++] = text_end;
	    right[pos] = '\0';
	  } else
	    found = TRUE;
	} while(*tmp);
	if(verbose && !found)
	  ses->print((const uchar *)"#Didn't find such word in that variable.\n\r");
      }
      delete ln;
    }
    ln = new listnode(myvars, left, right, TYPE_VAR, 5);
    if(IS_SET(ses->message_flags, MESSAGE_VAR) || verbose) {
      ses->print((const uchar *)"#OK. VARIABLE DEFINED.\n\r");
    }
  }
  return(ses);
}

/***************************/
/* the #unvariable command */
/***************************/
session *comm_unvariable(const unsigned char *arg, session *ses)
{
  unsigned char left[BUFFER_SIZE];
  int found;
  struct listnode **myvars, **mycolors, *ln, *next;

  myvars = &ses->variables;
  mycolors = &ses->colors;

  arg = get_arg_stop_spaces(arg, left);

  for(found = FALSE,ln = searchnode_list2(*myvars, left); ln;
                    ln = searchnode_list2(next, left)) {
    found = TRUE;
    next = ln->next;
    delete ln;
    if(IS_SET(ses->message_flags, MESSAGE_VAR))
      ses->print((const uchar *)"#OK. VARIABLE DELETED\n\r");
  }
  for(ln = searchnode_list2(*mycolors, left); ln;
      ln = searchnode_list2(next, left)) {
    found = TRUE;
    next = ln->next;
    delete ln;
    if(IS_SET(ses->message_flags, MESSAGE_VAR))
      ses->print((const uchar *)"#OK. COLOR VARIABLE DELETED\n\r");
  }
  if(!found)
    ses->print((const uchar *)"#THAT VARIABLE IS NOT DEFINED.\n\r");
  return(ses);
}
